MouseTargeting_Options = {};

MouseTargeting_Setup = {
	isTracking = false;
};

function MouseTargeting_OnLoad()
	this:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	this:RegisterEvent("VARIABLES_LOADED");
end

function MouseTargeting_OnEvent()
	if ( event == "UPDATE_MOUSEOVER_UNIT" ) then
		if ( UnitExists("mouseover") ) then
			if ( MouseTargeting_Setup.isTracking ) then
				MouseTargeting_MouseTargetingUpdate();
			end
		else
			MouseTargeting_Message("DBG: Unit does not exist", MOUSETARGETING_MSG_LEVEL_DEBUG);
		end
	end
	if ( event == "VARIABLES_LOADED" ) then
		for k, v in MOUSETARGETING_OPTIONS_DEFAULT do
			if ( MouseTargeting_Options[k] == nil ) then
				MouseTargeting_Options[k] = v;
			end
		end
	end
end

function MouseTargeting_InitiateMouseTargeting()
	MouseTargeting_Message("InitiateMouseTargeting()", MOUSETARGETING_MSG_LEVEL_DEBUG);
	for k, v in MOUSETARGETING_SETUP_DEFAULT do
		MouseTargeting_Setup[k] = v;
	end
end

function MouseTargeting_MouseTargetingStart()
	if ( not MouseTargeting_Options.enabled ) then return; end
	MouseTargeting_Message("MouseTargetingStart()", MOUSETARGETING_MSG_LEVEL_DEBUG);
	MouseTargeting_InitiateMouseTargeting();
	MouseTargeting_Setup.isTracking = true;
end

function MouseTargeting_TargetHighestPriority()
	if ( not MouseTargeting_Options.enabled ) then return; end
	MouseTargeting_Message("TargetHighestPriority()", MOUSETARGETING_MSG_LEVEL_DEBUG);
	local name = MouseTargeting_Setup.highestPriorityName;
	if ( name ) and ( name ~= "" ) then
		MouseTargeting_Message("DBG: Targetting "..name, MOUSETARGETING_MSG_LEVEL_DEBUG);
		TargetByName(name);
		MouseTargeting_Message(format(MOUSETARGETING_MSG_TARGET_FOUND, name), MOUSETARGETING_MSG_LEVEL_INFO);
	else
		MouseTargeting_Message(MOUSETARGETING_MSG_NO_TARGET_FOUND, MOUSETARGETING_MSG_LEVEL_WARNING);
	end
end

function MouseTargeting_MouseTargetingStop()
	if ( not MouseTargeting_Options.enabled ) then return; end
	MouseTargeting_Message("MouseTargetingStop()", MOUSETARGETING_MSG_LEVEL_DEBUG);
	MouseTargeting_Setup.isTracking = false;
	MouseTargeting_TargetHighestPriority();
end

function MouseTargeting_Message(msg, level)
	if ( MouseTargeting_Options.verbosity ) and ( MouseTargeting_Options.verbosity >= level ) then
		local color = MOUSETARGETING_MSG_COLOR[level];
		if ( not color ) then color = MOUSETARGETING_MSG_COLOR.default; end
		ChatFrame1:AddMessage(msg, color.r, color.g, color.b, 1.0);
	end
end

function MouseTargeting_MouseTargetingUpdate()
	if ( not MouseTargeting_Options.enabled ) then return; end
	MouseTargeting_Message("MouseTargetingUpdate()", MOUSETARGETING_MSG_LEVEL_DEBUG);
	if ( not UnitIsEnemy("mouseover", "player") ) then
		MouseTargeting_Message("DBG: Abort: Unit not enemy", MOUSETARGETING_MSG_LEVEL_DEBUG);
		return;
	end
	local validType = MouseTargeting_Options.validTargetType;
	if ( validType ) and ( validType ~= MOUSETARGETING_TARGET_TYPE_ALL ) then
		local isPlayer = UnitIsPlayer("mouseover");
		if ( isPlayer ) and ( validType == MOUSETARGETING_TARGET_TYPE_NPC ) then
			MouseTargeting_Message("DBG: Abort: Unit was player", MOUSETARGETING_MSG_LEVEL_DEBUG);
			return;
		elseif ( not isPlayer ) and ( validType == MOUSETARGETING_TARGET_TYPE_PLAYER ) then
			MouseTargeting_Message("DBG: Abort: Unit was not player", MOUSETARGETING_MSG_LEVEL_DEBUG);
			return;
		end
	end
	if ( MouseTargeting_Options.onlyAlive ) then
		if ( UnitIsDead("mouseover") ) then
			MouseTargeting_Message("DBG: Abort: Unit was dead", MOUSETARGETING_MSG_LEVEL_DEBUG);
			return;
		end
	end
	local value = nil;
	local name = UnitName("mouseover");
	local level = UnitLevel("mouseover");
	if ( not name ) then
		return;
	end
	if ( level <= -1 ) then
		level = 61;
	end
	local mode = MouseTargeting_Options.mode;
	if ( not MouseTargeting_Options.priorities[mode] ) then
		MouseTargeting_Message("DBG: Abort: Priorities not found: "..mode, MOUSETARGETING_MSG_LEVEL_DEBUG);
		return;
	end
	if ( mode == "class" ) then
		local _, class = UnitClass("mouseover");
		if ( class ) then
			value = MouseTargeting_Options.priorities[mode][class];
		end
	end
	if ( value ) and ( value > MouseTargeting_Setup.highestPriorityValue ) and ( level > MouseTargeting_Setup.highestPriorityLevel )then
		MouseTargeting_Message("DBG: Chose new target "..name, MOUSETARGETING_MSG_LEVEL_DEBUG);
		MouseTargeting_Setup.highestPriorityValue = value;
		MouseTargeting_Setup.highestPriorityName = name;
		if ( MouseTargeting_Options.continualTargeting ) then
			MouseTargeting_TargetHighestPriority();
		end
	end
end